#include <Pilot.h>
#include <PalmOS3/UI/ScrDriverNew.h>
#include "befh.h"

typedef unsigned char PositionType;



static char default_excuse[] =
"Repeat after me: Next Time, I Won't Forget To Install "
"The Excuse Database (self-lart engaged).";




#define soloAppVersionNum 0x0002
#define soloAppCreator    CREATOR /* Makefile defines this */

#define BEFHDBType    'DATA'




static DmOpenRef     BEFHDB;

static int           num_excuses;



static void
display()
{
  RectangleType rect;
  char *cp;
  SWord xpos;
  SWord ypos = 40;
  char *current_excuse;
  VoidHand excuseRec;
  int random_excuse=0;

  rect.topLeft.x=0;
  rect.topLeft.y=18;
  rect.extent.x=160;
  rect.extent.y=120;
  WinEraseRectangle(&rect,1);
  FntSetFont(2);
  
  cp=default_excuse;

  current_excuse=NULL;
  if(!BEFHDB) cp=default_excuse;
  else {
    if(num_excuses<1) cp=default_excuse;
    else {
      random_excuse = (((UInt)SysRandom(0))>>3)%num_excuses;
      excuseRec=DmGetRecord(BEFHDB,random_excuse);
      if(!excuseRec) cp=default_excuse;
      else cp=current_excuse=(char*)MemHandleLock(excuseRec);
    }
  }

  ClipboardAddItem(clipboardText,cp,StrLen(cp));

  for(;*cp;){
    int delta=FntWordWrap(cp,rect.extent.x-10);
    xpos=(160-FntCharsWidth(cp,delta))>>1;
    WinDrawChars(cp,delta,xpos,ypos);
    ypos+=FntLineHeight();
    cp+=delta;
    if(*cp==0)break;
  }

  if(current_excuse){
    MemPtrUnlock(current_excuse);
    DmReleaseRecord(BEFHDB,random_excuse,true);
  }
}


static Boolean
MainFormHandleEvent(EventPtr event)
{
  Boolean handled=false;
  FormPtr frm;
  
  switch (event->eType) {
  case frmOpenEvent:
    frm = FrmGetActiveForm();
    FrmDrawForm(frm);
    handled = true;
    break;

  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID==ExcuseButton) {
      SndPlaySystemSound(sndClick);
      display();
      handled = true;
    }
    break;

  case penDownEvent:
    SndPlaySystemSound(sndClick);
    display();
    handled = true;
    break;
    
  case menuEvent:
    MenuEraseStatus(MenuGetActiveMenu());
    switch(event->data.menu.itemID){

    case MainHelpShowEther:
      {
	RectangleType rect;
	rect.topLeft.x=0;
	rect.topLeft.y=18;
	rect.extent.x=160;
	rect.extent.y=120;
	WinEraseRectangle(&rect,1);
	frm = FrmGetActiveForm();
	FrmDrawForm(frm);
	handled = true;
      }
      break;

    case MainHelpAbout:
      frm = FrmInitForm(AboutForm);
      FrmDoDialog(frm);
      FrmDeleteForm(frm);
      handled = true;
      break;
    }
    break;

  default:
    break;			/* To stop spurious warning */
  }
  return handled;
}


static Boolean
ApplicationHandleEvent(EventPtr event)
{
  FormPtr  frm;
  Int      formId;
  Boolean  handled=false;

  if (event->eType == frmLoadEvent) {
    formId=event->data.frmLoad.formID;
    frm=FrmInitForm(formId);
    FrmSetActiveForm(frm);
    
    switch(formId) {
    case MainForm:
      FrmSetEventHandler(frm,MainFormHandleEvent);
      break;
    }
    handled = true;
  }
  return handled;
}


static void
EventLoop()
{
  EventType event;
  Word      error;

  do {
    EvtGetEvent(&event, 2);
    if (!SysHandleEvent(&event)){
      if (!MenuHandleEvent(0, &event, &error)){
	if (!ApplicationHandleEvent(&event)){
	  FrmDispatchEvent(&event);
	}
      }
    }
  } while (event.eType!=appStopEvent);
}


static void
OpenPDB()
{
  BEFHDB=DmOpenDatabaseByTypeCreator(BEFHDBType, CREATOR, dmModeReadOnly);
  if(BEFHDB) num_excuses = DmNumRecords(BEFHDB);
}



static Boolean
StartApplication()
{
  OpenPDB();

  FrmGotoForm(MainForm);

  return false;
}


static void
StopApplication()
{
  FrmCloseAllForms();
  if(BEFHDB)DmCloseDatabase(BEFHDB); 
}


DWord
PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{

  if(cmd==sysAppLaunchCmdNormalLaunch) {
    if (!StartApplication()) {
      EventLoop();
      StopApplication();
    }
  }
  return 0;
}
